###############################################################################
# International Holiday Data provided by Holidata.net
# https://holidata.net/nl-NL/2020.json
# https://holidata.net/nl-NL/2021.json
#
# Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

holiday.nl-NL1.name=Nieuwjaarsdag
holiday.nl-NL1.date=20200101
holiday.nl-NL2.name=Goede Vrijdag
holiday.nl-NL2.date=20200410
holiday.nl-NL3.name=Eerste Paasdag
holiday.nl-NL3.date=20200412
holiday.nl-NL4.name=Tweede Paasdag
holiday.nl-NL4.date=20200413
holiday.nl-NL5.name=Koningsdag
holiday.nl-NL5.date=20200427
holiday.nl-NL6.name=Dodenherdenking
holiday.nl-NL6.date=20200504
holiday.nl-NL7.name=Bevrijdingsdag
holiday.nl-NL7.date=20200505
holiday.nl-NL8.name=Hemelvaartsdag
holiday.nl-NL8.date=20200521
holiday.nl-NL9.name=Eerste Pinksterdag
holiday.nl-NL9.date=20200531
holiday.nl-NL10.name=Tweede Pinksterdag
holiday.nl-NL10.date=20200601
holiday.nl-NL11.name=Sinterklaas
holiday.nl-NL11.date=20201205
holiday.nl-NL12.name=Koninkrijksdag
holiday.nl-NL12.date=20201215
holiday.nl-NL13.name=Eerste Kerstdag
holiday.nl-NL13.date=20201225
holiday.nl-NL14.name=Tweede Kerstdag
holiday.nl-NL14.date=20201226
holiday.nl-NL15.name=Nieuwjaarsdag
holiday.nl-NL15.date=20210101
holiday.nl-NL16.name=Goede Vrijdag
holiday.nl-NL16.date=20210402
holiday.nl-NL17.name=Eerste Paasdag
holiday.nl-NL17.date=20210404
holiday.nl-NL18.name=Tweede Paasdag
holiday.nl-NL18.date=20210405
holiday.nl-NL19.name=Koningsdag
holiday.nl-NL19.date=20210427
holiday.nl-NL20.name=Dodenherdenking
holiday.nl-NL20.date=20210504
holiday.nl-NL21.name=Bevrijdingsdag
holiday.nl-NL21.date=20210505
holiday.nl-NL22.name=Hemelvaartsdag
holiday.nl-NL22.date=20210513
holiday.nl-NL23.name=Eerste Pinksterdag
holiday.nl-NL23.date=20210523
holiday.nl-NL24.name=Tweede Pinksterdag
holiday.nl-NL24.date=20210524
holiday.nl-NL25.name=Sinterklaas
holiday.nl-NL25.date=20211205
holiday.nl-NL26.name=Koninkrijksdag
holiday.nl-NL26.date=20211215
holiday.nl-NL27.name=Eerste Kerstdag
holiday.nl-NL27.date=20211225
holiday.nl-NL28.name=Tweede Kerstdag
holiday.nl-NL28.date=20211226
