###############################################################################
# International Holiday Data provided by Holidata.net
# https://holidata.net/hr-HR/2020.json
# https://holidata.net/hr-HR/2021.json
#
# Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

holiday.hr-HR1.name=Nova Godina
holiday.hr-HR1.date=20200101
holiday.hr-HR2.name=Bogojavljenje
holiday.hr-HR2.date=20200106
holiday.hr-HR3.name=Uskrs
holiday.hr-HR3.date=20200412
holiday.hr-HR4.name=Uskrsni ponedjeljak
holiday.hr-HR4.date=20200413
holiday.hr-HR5.name=Praznik rada
holiday.hr-HR5.date=20200501
holiday.hr-HR6.name=Tijelovo
holiday.hr-HR6.date=20200611
holiday.hr-HR7.name=Dan antifašističke borbe
holiday.hr-HR7.date=20200622
holiday.hr-HR8.name=Dan državnosti
holiday.hr-HR8.date=20200625
holiday.hr-HR9.name=Dan pobjede i domovinske zahvalnosti i Dan hrvatskih branitelja
holiday.hr-HR9.date=20200805
holiday.hr-HR10.name=Velika Gospa
holiday.hr-HR10.date=20200815
holiday.hr-HR11.name=Dan neovisnosti
holiday.hr-HR11.date=20201008
holiday.hr-HR12.name=Svi sveti
holiday.hr-HR12.date=20201101
holiday.hr-HR13.name=Dan sjećanja na žrtve Domovinskog rata i Dan sjećanja na žrtvu Vukovara i Škabrnje
holiday.hr-HR13.date=20201118
holiday.hr-HR14.name=Božić
holiday.hr-HR14.date=20201225
holiday.hr-HR15.name=Sveti Stjepan
holiday.hr-HR15.date=20201226
holiday.hr-HR16.name=Nova Godina
holiday.hr-HR16.date=20210101
holiday.hr-HR17.name=Bogojavljenje
holiday.hr-HR17.date=20210106
holiday.hr-HR18.name=Uskrs
holiday.hr-HR18.date=20210404
holiday.hr-HR19.name=Uskrsni ponedjeljak
holiday.hr-HR19.date=20210405
holiday.hr-HR20.name=Praznik rada
holiday.hr-HR20.date=20210501
holiday.hr-HR21.name=Tijelovo
holiday.hr-HR21.date=20210603
holiday.hr-HR22.name=Dan antifašističke borbe
holiday.hr-HR22.date=20210622
holiday.hr-HR23.name=Dan državnosti
holiday.hr-HR23.date=20210625
holiday.hr-HR24.name=Dan pobjede i domovinske zahvalnosti i Dan hrvatskih branitelja
holiday.hr-HR24.date=20210805
holiday.hr-HR25.name=Velika Gospa
holiday.hr-HR25.date=20210815
holiday.hr-HR26.name=Dan neovisnosti
holiday.hr-HR26.date=20211008
holiday.hr-HR27.name=Svi sveti
holiday.hr-HR27.date=20211101
holiday.hr-HR28.name=Dan sjećanja na žrtve Domovinskog rata i Dan sjećanja na žrtvu Vukovara i Škabrnje
holiday.hr-HR28.date=20211118
holiday.hr-HR29.name=Božić
holiday.hr-HR29.date=20211225
holiday.hr-HR30.name=Sveti Stjepan
holiday.hr-HR30.date=20211226
