###############################################################################
# International Holiday Data provided by Holidata.net
# https://holidata.net/tr-TR/2020.json
# https://holidata.net/tr-TR/2021.json
#
# Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

holiday.tr-TR1.name=Yılbaşı
holiday.tr-TR1.date=20200101
holiday.tr-TR2.name=Ulusal Egemenlik ve Çocuk Bayramı
holiday.tr-TR2.date=20200423
holiday.tr-TR3.name=Emek ve Dayanışma Günü
holiday.tr-TR3.date=20200501
holiday.tr-TR4.name=Atatürk'ü Anma, Gençlik ve Spor Bayramı
holiday.tr-TR4.date=20200519
holiday.tr-TR5.name=Ramazan Bayramı (1. Gün)
holiday.tr-TR5.date=20200524
holiday.tr-TR6.name=Ramazan Bayramı (2. Gün)
holiday.tr-TR6.date=20200525
holiday.tr-TR7.name=Ramazan Bayramı (3. Gün)
holiday.tr-TR7.date=20200526
holiday.tr-TR8.name=Demokrasi ve Milli Birlik Günü
holiday.tr-TR8.date=20200715
holiday.tr-TR9.name=Kurban Bayramı (1. Gün)
holiday.tr-TR9.date=20200731
holiday.tr-TR10.name=Kurban Bayramı (2. Gün)
holiday.tr-TR10.date=20200801
holiday.tr-TR11.name=Kurban Bayramı (3. Gün)
holiday.tr-TR11.date=20200802
holiday.tr-TR12.name=Kurban Bayramı (4. Gün)
holiday.tr-TR12.date=20200803
holiday.tr-TR13.name=Zafer Bayramı
holiday.tr-TR13.date=20200830
holiday.tr-TR14.name=Cumhuriyet Bayramı
holiday.tr-TR14.date=20201029
holiday.tr-TR15.name=Yılbaşı
holiday.tr-TR15.date=20210101
holiday.tr-TR16.name=Ulusal Egemenlik ve Çocuk Bayramı
holiday.tr-TR16.date=20210423
holiday.tr-TR17.name=Emek ve Dayanışma Günü
holiday.tr-TR17.date=20210501
holiday.tr-TR18.name=Ramazan Bayramı (1. Gün)
holiday.tr-TR18.date=20210513
holiday.tr-TR19.name=Ramazan Bayramı (2. Gün)
holiday.tr-TR19.date=20210514
holiday.tr-TR20.name=Ramazan Bayramı (3. Gün)
holiday.tr-TR20.date=20210515
holiday.tr-TR21.name=Atatürk'ü Anma, Gençlik ve Spor Bayramı
holiday.tr-TR21.date=20210519
holiday.tr-TR22.name=Demokrasi ve Milli Birlik Günü
holiday.tr-TR22.date=20210715
holiday.tr-TR23.name=Kurban Bayramı (1. Gün)
holiday.tr-TR23.date=20210720
holiday.tr-TR24.name=Kurban Bayramı (2. Gün)
holiday.tr-TR24.date=20210721
holiday.tr-TR25.name=Kurban Bayramı (3. Gün)
holiday.tr-TR25.date=20210722
holiday.tr-TR26.name=Kurban Bayramı (4. Gün)
holiday.tr-TR26.date=20210723
holiday.tr-TR27.name=Zafer Bayramı
holiday.tr-TR27.date=20210830
holiday.tr-TR28.name=Cumhuriyet Bayramı
holiday.tr-TR28.date=20211029
