#!/usr/bin/env python3
# -*- coding: utf-8 -*-
###############################################################################
#
# Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

import sys
import os
import unittest
# Ensure python finds the local simpletap module
sys.path.append(os.path.dirname(os.path.abspath(__file__)))

from basetest import Task, TestCase


class TestArgs(TestCase):
    def setUp(self):
        """Executed before each test in the class"""
        self.t = Task()

    def test_dash_dash_argument(self):
        """Test the -- argument"""
        self.t("add project:p pri:H +tag foo")

        code, out, err = self.t("_get 1.description")
        self.assertIn("foo\n", out, msg='add project:p pri:H +tag foo')

        self.t("1 modify project:p pri:H +tag -- foo")
        code, out, err = self.t("_get 1.description")
        self.assertIn("foo\n", out, msg='add project:p pri:H +tag -- foo')

        self.t("1 modify project:p pri:H -- +tag foo")
        code, out, err = self.t("_get 1.description")
        self.assertIn("+tag foo\n", out, msg='add project:p pri:H -- +tag foo')

        self.t("1 modify project:p -- pri:H +tag foo")
        code, out, err = self.t("_get 1.description")
        self.assertIn("pri:H +tag foo\n", out, msg='add project:p -- pri:H +tag foo')

        self.t("1 modify -- project:p pri:H +tag foo")
        code, out, err = self.t("_get 1.description")
        self.assertIn("project:p pri:H +tag foo\n", out, msg='add -- project:p pri:H +tag foo')

        self.t("1 modify -- project:p pri:H +tag foo --")
        code, out, err = self.t("_get 1.description")
        self.assertIn("project:p pri:H +tag foo --\n", out, msg='add -- project:p pri:H +tag foo --')


class TestIDPosition(TestCase):
    @classmethod
    def setUpClass(cls):
        """Executed once before any test in the class"""
        cls.t = Task()

        cls.t("add one")
        cls.t("add two")

    def test_id_read_cmd(self):
        """Test id before and after read command"""
        code, out, err = self.t("1 info")
        self.assertIn("one", out)
        self.assertNotIn("two", out)

        code, out, err = self.t("info 1")
        self.assertIn("one", out)
        self.assertNotIn("two", out)

    def test_id_write_cmd(self):
        """Test id before write command"""
        code, out, err = self.t("2 done")
        self.assertIn("Completed task 2", out)


if __name__ == "__main__":
    from simpletap import TAPTestRunner
    unittest.main(testRunner=TAPTestRunner())

# vim: ai sts=4 et sw=4 ft=python
