###############################################################################
# International Holiday Data provided by Holidata.net
# https://holidata.net/de-DE/2020.json
# https://holidata.net/de-DE/2021.json
#
# Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

holiday.de-DE1.name=Neujahr
holiday.de-DE1.date=20200101
holiday.de-DE2.name=Heilige drei Könige
holiday.de-DE2.date=20200106
holiday.de-DE3.name=Internationaler Frauentag
holiday.de-DE3.date=20200308
holiday.de-DE4.name=Karfreitag
holiday.de-DE4.date=20200410
holiday.de-DE5.name=Ostern
holiday.de-DE5.date=20200412
holiday.de-DE6.name=Ostermontag
holiday.de-DE6.date=20200413
holiday.de-DE7.name=Erster Maifeiertag
holiday.de-DE7.date=20200501
holiday.de-DE8.name=75. Jahrestag der Befreiung vom Nationalsozialismus und der Beendigung des Zweiten Weltkrieges in Europa
holiday.de-DE8.date=20200508
holiday.de-DE9.name=Christi Himmelfahrt
holiday.de-DE9.date=20200521
holiday.de-DE10.name=Pfingstsonntag
holiday.de-DE10.date=20200531
holiday.de-DE11.name=Pfingstmontag
holiday.de-DE11.date=20200601
holiday.de-DE12.name=Fronleichnam
holiday.de-DE12.date=20200611
holiday.de-DE13.name=Mariä Himmelfahrt
holiday.de-DE13.date=20200815
holiday.de-DE14.name=Tag der Deutschen Einheit
holiday.de-DE14.date=20201003
holiday.de-DE15.name=Reformationstag
holiday.de-DE15.date=20201031
holiday.de-DE16.name=Allerheiligen
holiday.de-DE16.date=20201101
holiday.de-DE17.name=Buß- und Bettag
holiday.de-DE17.date=20201118
holiday.de-DE18.name=Heilig Abend
holiday.de-DE18.date=20201224
holiday.de-DE19.name=Weihnachtstag
holiday.de-DE19.date=20201225
holiday.de-DE20.name=Zweiter Weihnachtstag
holiday.de-DE20.date=20201226
holiday.de-DE21.name=Silvester
holiday.de-DE21.date=20201231
holiday.de-DE22.name=Neujahr
holiday.de-DE22.date=20210101
holiday.de-DE23.name=Heilige drei Könige
holiday.de-DE23.date=20210106
holiday.de-DE24.name=Internationaler Frauentag
holiday.de-DE24.date=20210308
holiday.de-DE25.name=Karfreitag
holiday.de-DE25.date=20210402
holiday.de-DE26.name=Ostern
holiday.de-DE26.date=20210404
holiday.de-DE27.name=Ostermontag
holiday.de-DE27.date=20210405
holiday.de-DE28.name=Erster Maifeiertag
holiday.de-DE28.date=20210501
holiday.de-DE29.name=Christi Himmelfahrt
holiday.de-DE29.date=20210513
holiday.de-DE30.name=Pfingstsonntag
holiday.de-DE30.date=20210523
holiday.de-DE31.name=Pfingstmontag
holiday.de-DE31.date=20210524
holiday.de-DE32.name=Fronleichnam
holiday.de-DE32.date=20210603
holiday.de-DE33.name=Mariä Himmelfahrt
holiday.de-DE33.date=20210815
holiday.de-DE34.name=Tag der Deutschen Einheit
holiday.de-DE34.date=20211003
holiday.de-DE35.name=Reformationstag
holiday.de-DE35.date=20211031
holiday.de-DE36.name=Allerheiligen
holiday.de-DE36.date=20211101
holiday.de-DE37.name=Buß- und Bettag
holiday.de-DE37.date=20211117
holiday.de-DE38.name=Heilig Abend
holiday.de-DE38.date=20211224
holiday.de-DE39.name=Weihnachtstag
holiday.de-DE39.date=20211225
holiday.de-DE40.name=Zweiter Weihnachtstag
holiday.de-DE40.date=20211226
holiday.de-DE41.name=Silvester
holiday.de-DE41.date=20211231
