###############################################################################
# International Holiday Data provided by Holidata.net
# https://holidata.net/en-GB/2020.json
# https://holidata.net/en-GB/2021.json
#
# Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

holiday.en-GB1.name=New Year's Day
holiday.en-GB1.date=20200101
holiday.en-GB2.name=Good Friday
holiday.en-GB2.date=20200410
holiday.en-GB3.name=Easter Monday
holiday.en-GB3.date=20200413
holiday.en-GB4.name=Early May Bank Holiday
holiday.en-GB4.date=20200504
holiday.en-GB5.name=Spring Bank Holiday
holiday.en-GB5.date=20200525
holiday.en-GB6.name=August Bank Holiday
holiday.en-GB6.date=20200831
holiday.en-GB7.name=Christmas Day
holiday.en-GB7.date=20201225
holiday.en-GB8.name=Boxing Day
holiday.en-GB8.date=20201226
holiday.en-GB9.name=Boxing Day (observed)
holiday.en-GB9.date=20201228
holiday.en-GB10.name=New Year's Day
holiday.en-GB10.date=20210101
holiday.en-GB11.name=Good Friday
holiday.en-GB11.date=20210402
holiday.en-GB12.name=Easter Monday
holiday.en-GB12.date=20210405
holiday.en-GB13.name=Early May Bank Holiday
holiday.en-GB13.date=20210503
holiday.en-GB14.name=Spring Bank Holiday
holiday.en-GB14.date=20210531
holiday.en-GB15.name=August Bank Holiday
holiday.en-GB15.date=20210830
holiday.en-GB16.name=Christmas Day
holiday.en-GB16.date=20211225
holiday.en-GB17.name=Boxing Day
holiday.en-GB17.date=20211226
holiday.en-GB18.name=Christmas Day (observed)
holiday.en-GB18.date=20211227
holiday.en-GB19.name=Boxing Day (observed)
holiday.en-GB19.date=20211228
