###############################################################################
# International Holiday Data provided by Holidata.net
# https://holidata.net/es-ES/2020.json
# https://holidata.net/es-ES/2021.json
#
# Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

holiday.es-ES1.name=Año Nuevo
holiday.es-ES1.date=20200101
holiday.es-ES2.name=Día de los Reyes
holiday.es-ES2.date=20200106
holiday.es-ES3.name=Viernes Santo
holiday.es-ES3.date=20200410
holiday.es-ES4.name=Pascua
holiday.es-ES4.date=20200412
holiday.es-ES5.name=Fiesta del Trabajo
holiday.es-ES5.date=20200501
holiday.es-ES6.name=Asunción de la Virgen
holiday.es-ES6.date=20200815
holiday.es-ES7.name=Fiesta Nacional de España
holiday.es-ES7.date=20201012
holiday.es-ES8.name=Dia de todos los Santos
holiday.es-ES8.date=20201101
holiday.es-ES9.name=Dia de la Constitución
holiday.es-ES9.date=20201206
holiday.es-ES10.name=Inmaculada Concepción
holiday.es-ES10.date=20201208
holiday.es-ES11.name=Noche Buena
holiday.es-ES11.date=20201224
holiday.es-ES12.name=Navidad
holiday.es-ES12.date=20201225
holiday.es-ES13.name=Noche Vieja
holiday.es-ES13.date=20201231
holiday.es-ES14.name=Año Nuevo
holiday.es-ES14.date=20210101
holiday.es-ES15.name=Día de los Reyes
holiday.es-ES15.date=20210106
holiday.es-ES16.name=Viernes Santo
holiday.es-ES16.date=20210402
holiday.es-ES17.name=Pascua
holiday.es-ES17.date=20210404
holiday.es-ES18.name=Fiesta del Trabajo
holiday.es-ES18.date=20210501
holiday.es-ES19.name=Asunción de la Virgen
holiday.es-ES19.date=20210815
holiday.es-ES20.name=Fiesta Nacional de España
holiday.es-ES20.date=20211012
holiday.es-ES21.name=Dia de todos los Santos
holiday.es-ES21.date=20211101
holiday.es-ES22.name=Dia de la Constitución
holiday.es-ES22.date=20211206
holiday.es-ES23.name=Inmaculada Concepción
holiday.es-ES23.date=20211208
holiday.es-ES24.name=Noche Buena
holiday.es-ES24.date=20211224
holiday.es-ES25.name=Navidad
holiday.es-ES25.date=20211225
holiday.es-ES26.name=Noche Vieja
holiday.es-ES26.date=20211231
