###############################################################################
# International Holiday Data provided by Holidata.net
# https://holidata.net/es-US/2020.json
# https://holidata.net/es-US/2021.json
#
# Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

holiday.es-US1.name=Año Neuvo
holiday.es-US1.date=20200101
holiday.es-US2.name=Cumpleaños de Martin Luther King, Jr.
holiday.es-US2.date=20200120
holiday.es-US3.name=Día del Presidente
holiday.es-US3.date=20200217
holiday.es-US4.name=Día del Patriota
holiday.es-US4.date=20200420
holiday.es-US5.name=Día de los Caídos
holiday.es-US5.date=20200525
holiday.es-US6.name=Día de la Independiencia
holiday.es-US6.date=20200704
holiday.es-US7.name=Día del Trabajo
holiday.es-US7.date=20200907
holiday.es-US8.name=Día de Columbus
holiday.es-US8.date=20201012
holiday.es-US9.name=Día de los Veteranos
holiday.es-US9.date=20201111
holiday.es-US10.name=Día de Acción de Gracias
holiday.es-US10.date=20201126
holiday.es-US11.name=Día después de Acción de Gracias
holiday.es-US11.date=20201127
holiday.es-US12.name=Nochebuena
holiday.es-US12.date=20201224
holiday.es-US13.name=Navidad
holiday.es-US13.date=20201225
holiday.es-US14.name=Año Neuvo
holiday.es-US14.date=20210101
holiday.es-US15.name=Cumpleaños de Martin Luther King, Jr.
holiday.es-US15.date=20210118
holiday.es-US16.name=Día del Presidente
holiday.es-US16.date=20210215
holiday.es-US17.name=Día del Patriota
holiday.es-US17.date=20210419
holiday.es-US18.name=Día de los Caídos
holiday.es-US18.date=20210531
holiday.es-US19.name=Día de la Independiencia
holiday.es-US19.date=20210704
holiday.es-US20.name=Día del Trabajo
holiday.es-US20.date=20210906
holiday.es-US21.name=Día de Columbus
holiday.es-US21.date=20211011
holiday.es-US22.name=Día de los Veteranos
holiday.es-US22.date=20211111
holiday.es-US23.name=Día de Acción de Gracias
holiday.es-US23.date=20211125
holiday.es-US24.name=Navidad
holiday.es-US24.date=20211225
