###############################################################################
# International Holiday Data provided by Holidata.net
# https://holidata.net/de-AT/2020.json
# https://holidata.net/de-AT/2021.json
#
# Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

holiday.de-AT1.name=Neujahr
holiday.de-AT1.date=20200101
holiday.de-AT2.name=Heilige drei Könige
holiday.de-AT2.date=20200106
holiday.de-AT3.name=Josef
holiday.de-AT3.date=20200319
holiday.de-AT4.name=Karfreitag
holiday.de-AT4.date=20200410
holiday.de-AT5.name=Ostersonntag
holiday.de-AT5.date=20200412
holiday.de-AT6.name=Ostermontag
holiday.de-AT6.date=20200413
holiday.de-AT7.name=Staatsfeiertag
holiday.de-AT7.date=20200501
holiday.de-AT8.name=Florian
holiday.de-AT8.date=20200504
holiday.de-AT9.name=Christi Himmelfahrt
holiday.de-AT9.date=20200521
holiday.de-AT10.name=Pfingstsonntag
holiday.de-AT10.date=20200531
holiday.de-AT11.name=Pfingstmontag
holiday.de-AT11.date=20200601
holiday.de-AT12.name=Fronleichnam
holiday.de-AT12.date=20200611
holiday.de-AT13.name=Mariä Himmelfahrt
holiday.de-AT13.date=20200815
holiday.de-AT14.name=Rupert
holiday.de-AT14.date=20200924
holiday.de-AT15.name=Tag der Volksabstimmung
holiday.de-AT15.date=20201010
holiday.de-AT16.name=Nationalfeiertag
holiday.de-AT16.date=20201026
holiday.de-AT17.name=Allerheiligen
holiday.de-AT17.date=20201101
holiday.de-AT18.name=Martin
holiday.de-AT18.date=20201111
holiday.de-AT19.name=Leopold
holiday.de-AT19.date=20201115
holiday.de-AT20.name=Mariä Empfängnis
holiday.de-AT20.date=20201208
holiday.de-AT21.name=Heiliger Abend
holiday.de-AT21.date=20201224
holiday.de-AT22.name=Christtag
holiday.de-AT22.date=20201225
holiday.de-AT23.name=Stefanitag
holiday.de-AT23.date=20201226
holiday.de-AT24.name=Silvester
holiday.de-AT24.date=20201231
holiday.de-AT25.name=Neujahr
holiday.de-AT25.date=20210101
holiday.de-AT26.name=Heilige drei Könige
holiday.de-AT26.date=20210106
holiday.de-AT27.name=Josef
holiday.de-AT27.date=20210319
holiday.de-AT28.name=Karfreitag
holiday.de-AT28.date=20210402
holiday.de-AT29.name=Ostersonntag
holiday.de-AT29.date=20210404
holiday.de-AT30.name=Ostermontag
holiday.de-AT30.date=20210405
holiday.de-AT31.name=Staatsfeiertag
holiday.de-AT31.date=20210501
holiday.de-AT32.name=Florian
holiday.de-AT32.date=20210504
holiday.de-AT33.name=Christi Himmelfahrt
holiday.de-AT33.date=20210513
holiday.de-AT34.name=Pfingstsonntag
holiday.de-AT34.date=20210523
holiday.de-AT35.name=Pfingstmontag
holiday.de-AT35.date=20210524
holiday.de-AT36.name=Fronleichnam
holiday.de-AT36.date=20210603
holiday.de-AT37.name=Mariä Himmelfahrt
holiday.de-AT37.date=20210815
holiday.de-AT38.name=Rupert
holiday.de-AT38.date=20210924
holiday.de-AT39.name=Tag der Volksabstimmung
holiday.de-AT39.date=20211010
holiday.de-AT40.name=Nationalfeiertag
holiday.de-AT40.date=20211026
holiday.de-AT41.name=Allerheiligen
holiday.de-AT41.date=20211101
holiday.de-AT42.name=Martin
holiday.de-AT42.date=20211111
holiday.de-AT43.name=Leopold
holiday.de-AT43.date=20211115
holiday.de-AT44.name=Mariä Empfängnis
holiday.de-AT44.date=20211208
holiday.de-AT45.name=Heiliger Abend
holiday.de-AT45.date=20211224
holiday.de-AT46.name=Christtag
holiday.de-AT46.date=20211225
holiday.de-AT47.name=Stefanitag
holiday.de-AT47.date=20211226
holiday.de-AT48.name=Silvester
holiday.de-AT48.date=20211231
