###############################################################################
# International Holiday Data provided by Holidata.net
# https://holidata.net/pl-PL/2020.json
# https://holidata.net/pl-PL/2021.json
#
# Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

holiday.pl-PL1.name=Nowy Rok
holiday.pl-PL1.date=20200101
holiday.pl-PL2.name=Trzech Króli
holiday.pl-PL2.date=20200106
holiday.pl-PL3.name=Wielkanoc
holiday.pl-PL3.date=20200412
holiday.pl-PL4.name=Poniedziałek Wielkanocny
holiday.pl-PL4.date=20200413
holiday.pl-PL5.name=Święto Pracy
holiday.pl-PL5.date=20200501
holiday.pl-PL6.name=Święto Konstytucji Trzeciego Maja
holiday.pl-PL6.date=20200503
holiday.pl-PL7.name=Zielone Świątki
holiday.pl-PL7.date=20200531
holiday.pl-PL8.name=Boże Ciało
holiday.pl-PL8.date=20200611
holiday.pl-PL9.name=Wniebowzięcie Najświętszej Maryi Panny
holiday.pl-PL9.date=20200815
holiday.pl-PL10.name=Wszystkich Świętych
holiday.pl-PL10.date=20201101
holiday.pl-PL11.name=Narodowe Święto Niepodległości
holiday.pl-PL11.date=20201111
holiday.pl-PL12.name=Boże Narodzenie (pierwszy dzień)
holiday.pl-PL12.date=20201225
holiday.pl-PL13.name=Boże Narodzenie (drugi dzień)
holiday.pl-PL13.date=20201226
holiday.pl-PL14.name=Nowy Rok
holiday.pl-PL14.date=20210101
holiday.pl-PL15.name=Trzech Króli
holiday.pl-PL15.date=20210106
holiday.pl-PL16.name=Wielkanoc
holiday.pl-PL16.date=20210404
holiday.pl-PL17.name=Poniedziałek Wielkanocny
holiday.pl-PL17.date=20210405
holiday.pl-PL18.name=Święto Pracy
holiday.pl-PL18.date=20210501
holiday.pl-PL19.name=Święto Konstytucji Trzeciego Maja
holiday.pl-PL19.date=20210503
holiday.pl-PL20.name=Zielone Świątki
holiday.pl-PL20.date=20210523
holiday.pl-PL21.name=Boże Ciało
holiday.pl-PL21.date=20210603
holiday.pl-PL22.name=Wniebowzięcie Najświętszej Maryi Panny
holiday.pl-PL22.date=20210815
holiday.pl-PL23.name=Wszystkich Świętych
holiday.pl-PL23.date=20211101
holiday.pl-PL24.name=Narodowe Święto Niepodległości
holiday.pl-PL24.date=20211111
holiday.pl-PL25.name=Boże Narodzenie (pierwszy dzień)
holiday.pl-PL25.date=20211225
holiday.pl-PL26.name=Boże Narodzenie (drugi dzień)
holiday.pl-PL26.date=20211226
