###############################################################################
# International Holiday Data provided by Holidata.net
# https://holidata.net/nl-BE/2020.json
# https://holidata.net/nl-BE/2021.json
#
# Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

holiday.nl-BE1.name=Nieuwjaar
holiday.nl-BE1.date=20200101
holiday.nl-BE2.name=Pasen
holiday.nl-BE2.date=20200412
holiday.nl-BE3.name=Paasmaandag
holiday.nl-BE3.date=20200413
holiday.nl-BE4.name=Dag van de arbeid
holiday.nl-BE4.date=20200501
holiday.nl-BE5.name=Onze Lieve Heer hemelvaart
holiday.nl-BE5.date=20200521
holiday.nl-BE6.name=Pinksteren
holiday.nl-BE6.date=20200531
holiday.nl-BE7.name=Pinkstermaandag
holiday.nl-BE7.date=20200601
holiday.nl-BE8.name=Nationale feestdag
holiday.nl-BE8.date=20200721
holiday.nl-BE9.name=Onze Lieve Vrouw hemelvaart
holiday.nl-BE9.date=20200815
holiday.nl-BE10.name=Allerheiligen
holiday.nl-BE10.date=20201101
holiday.nl-BE11.name=Wapenstilstand
holiday.nl-BE11.date=20201111
holiday.nl-BE12.name=Kerstmis
holiday.nl-BE12.date=20201225
holiday.nl-BE13.name=Nieuwjaar
holiday.nl-BE13.date=20210101
holiday.nl-BE14.name=Pasen
holiday.nl-BE14.date=20210404
holiday.nl-BE15.name=Paasmaandag
holiday.nl-BE15.date=20210405
holiday.nl-BE16.name=Dag van de arbeid
holiday.nl-BE16.date=20210501
holiday.nl-BE17.name=Onze Lieve Heer hemelvaart
holiday.nl-BE17.date=20210513
holiday.nl-BE18.name=Pinksteren
holiday.nl-BE18.date=20210523
holiday.nl-BE19.name=Pinkstermaandag
holiday.nl-BE19.date=20210524
holiday.nl-BE20.name=Nationale feestdag
holiday.nl-BE20.date=20210721
holiday.nl-BE21.name=Onze Lieve Vrouw hemelvaart
holiday.nl-BE21.date=20210815
holiday.nl-BE22.name=Allerheiligen
holiday.nl-BE22.date=20211101
holiday.nl-BE23.name=Wapenstilstand
holiday.nl-BE23.date=20211111
holiday.nl-BE24.name=Kerstmis
holiday.nl-BE24.date=20211225
