###############################################################################
# International Holiday Data provided by Holidata.net
# https://holidata.net/cs-CZ/2020.json
# https://holidata.net/cs-CZ/2021.json
#
# Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

holiday.cs-CZ1.name=Nový rok
holiday.cs-CZ1.date=20200101
holiday.cs-CZ2.name=Den obnovy samostatného českého státu
holiday.cs-CZ2.date=20200101
holiday.cs-CZ3.name=Velký pátek
holiday.cs-CZ3.date=20200410
holiday.cs-CZ4.name=Velikonoční pondělí
holiday.cs-CZ4.date=20200413
holiday.cs-CZ5.name=Svátek práce
holiday.cs-CZ5.date=20200501
holiday.cs-CZ6.name=Den vítězství
holiday.cs-CZ6.date=20200508
holiday.cs-CZ7.name=Den slovanských věrozvěstů Cyrila a Metoděje
holiday.cs-CZ7.date=20200705
holiday.cs-CZ8.name=Den upálení mistra Jana Husa
holiday.cs-CZ8.date=20200706
holiday.cs-CZ9.name=Den české státnosti
holiday.cs-CZ9.date=20200928
holiday.cs-CZ10.name=Den vzniku samostatného československého státu
holiday.cs-CZ10.date=20201028
holiday.cs-CZ11.name=Den boje za svobodu a demokracii a Mezinárodní den studentstva
holiday.cs-CZ11.date=20201117
holiday.cs-CZ12.name=Štědrý den
holiday.cs-CZ12.date=20201224
holiday.cs-CZ13.name=1. svátek vánoční
holiday.cs-CZ13.date=20201225
holiday.cs-CZ14.name=2. svátek vánoční
holiday.cs-CZ14.date=20201226
holiday.cs-CZ15.name=Nový rok
holiday.cs-CZ15.date=20210101
holiday.cs-CZ16.name=Den obnovy samostatného českého státu
holiday.cs-CZ16.date=20210101
holiday.cs-CZ17.name=Velký pátek
holiday.cs-CZ17.date=20210402
holiday.cs-CZ18.name=Velikonoční pondělí
holiday.cs-CZ18.date=20210405
holiday.cs-CZ19.name=Svátek práce
holiday.cs-CZ19.date=20210501
holiday.cs-CZ20.name=Den vítězství
holiday.cs-CZ20.date=20210508
holiday.cs-CZ21.name=Den slovanských věrozvěstů Cyrila a Metoděje
holiday.cs-CZ21.date=20210705
holiday.cs-CZ22.name=Den upálení mistra Jana Husa
holiday.cs-CZ22.date=20210706
holiday.cs-CZ23.name=Den české státnosti
holiday.cs-CZ23.date=20210928
holiday.cs-CZ24.name=Den vzniku samostatného československého státu
holiday.cs-CZ24.date=20211028
holiday.cs-CZ25.name=Den boje za svobodu a demokracii a Mezinárodní den studentstva
holiday.cs-CZ25.date=20211117
holiday.cs-CZ26.name=Štědrý den
holiday.cs-CZ26.date=20211224
holiday.cs-CZ27.name=1. svátek vánoční
holiday.cs-CZ27.date=20211225
holiday.cs-CZ28.name=2. svátek vánoční
holiday.cs-CZ28.date=20211226
