###############################################################################
# International Holiday Data provided by Holidata.net
# https://holidata.net/ru-RU/2020.json
# https://holidata.net/ru-RU/2021.json
#
# Copyright 2006 - 2021, Paul Beckingham, Federico Hernandez.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# https://www.opensource.org/licenses/mit-license.php
#
###############################################################################

holiday.ru-RU1.name=Новый Год
holiday.ru-RU1.date=20200101
holiday.ru-RU2.name=Рождество Христово
holiday.ru-RU2.date=20200107
holiday.ru-RU3.name=День защитника Отечества
holiday.ru-RU3.date=20200223
holiday.ru-RU4.name=Международный женский день
holiday.ru-RU4.date=20200308
holiday.ru-RU5.name=Пасха
holiday.ru-RU5.date=20200419
holiday.ru-RU6.name=Праздник весны и труда
holiday.ru-RU6.date=20200501
holiday.ru-RU7.name=День Победы
holiday.ru-RU7.date=20200509
holiday.ru-RU8.name=День России
holiday.ru-RU8.date=20200612
holiday.ru-RU9.name=День народного единства
holiday.ru-RU9.date=20201104
holiday.ru-RU10.name=Новый Год
holiday.ru-RU10.date=20210101
holiday.ru-RU11.name=Рождество Христово
holiday.ru-RU11.date=20210107
holiday.ru-RU12.name=День защитника Отечества
holiday.ru-RU12.date=20210223
holiday.ru-RU13.name=Международный женский день
holiday.ru-RU13.date=20210308
holiday.ru-RU14.name=Праздник весны и труда
holiday.ru-RU14.date=20210501
holiday.ru-RU15.name=Пасха
holiday.ru-RU15.date=20210502
holiday.ru-RU16.name=День Победы
holiday.ru-RU16.date=20210509
holiday.ru-RU17.name=День России
holiday.ru-RU17.date=20210612
holiday.ru-RU18.name=День народного единства
holiday.ru-RU18.date=20211104
